import React from 'react';
import { Provider } from 'react-redux';
import {
  applyMiddleware,
  combineReducers,
  compose,
  createStore,
} from 'redux';
import {
  initializeCurrentLocation,
  Fragment,
  routerForBrowser,
} from 'redux-little-router';

const tree = routes => { console.log(Object.keys(routes)); return (
  <div>
    {Object.keys(routes).map((route, i) => (
      <Fragment
        forRoute={route}
        key={i}
        children={typeof routes[route] === 'function' ?
          React.createElement(routes[route]) :
          tree(routes[route])}
      />
    ))}
  </div>
)};

const hydrazine = ({
  basename,
  middlewares = [],
  reducers = new Object(),
  routes,
}) => {
  const {
    reducer: routerReducer,
    middleware: routerMiddleware,
    enhancer: routerEnhancer,
  } = routerForBrowser({
    basename,
    routes,
  });

  const store = createStore(
    combineReducers({ router: routerReducer, ...reducers }),
    undefined,
    compose(routerEnhancer, applyMiddleware(routerMiddleware, ...middlewares)),
  );

  const initialLocation = store.getState().router;
  if (initialLocation) {
    store.dispatch(initializeCurrentLocation(initialLocation));
  }

  const App = () => (
    <Provider
      store={store}
      children={tree(routes)}
    />
  );

  return {
    App,
    store,
  };
};

export default hydrazine;

/*
    this.builder = {
      allRoutes: [],
      enterListeners: {},
      layouts: {},
      leaveListeners: {},
      middlewares,
      reducers,
    };
    this.mountNode = mountNode;
  }

  start() {
    const {
      allRoutes,
      enterListeners,
      layouts,
      leaveListeners,
      middlewares,
      reducers,
    } = this.builder;

    const routes = {};
    allRoutes.forEach(route => {
      routes[route] = {};
    });

    const {
      reducer: routerReducer,
      middleware: routerMiddleware,
      enhancer: routerEnhancer,
    } = routerForBrowser({ routes });

    this.store = createStore(
      combineReducers({ router: routerReducer, ...reducers }),
      undefined,
      compose(routerEnhancer, applyMiddleware(routerMiddleware, ...middlewares)),
    );

    const initialLocation = this.store.getState().router;
    if (initialLocation) {
      this.store.dispatch(
        initializeCurrentLocation(initialLocation)
      );
    }

    console.log(Object.values(layouts).map(({ routes }) => routes));

    render((
      <Provider store={this.store}>
        <Fragment forRoute="/">
          <div>
            {Object.values(layouts).map((layout, i) => layout.routes.map((route, j) => (
              <Fragment
                forRoute={route}
                key={`${i}_${j}`}
                children={React.createElement(layout.component)}
              />
            )))}
          </div>
        </Fragment>
      </Provider>
    ), this.mountNode);

    delete this.builder;
  }
}

export default Hydrazine;*/
